﻿//
// FormMain.cs	METC Monitor メインフォーム
//				Copyright(C)2013-2017 Kashi Electronics Designs
//				kashima@kaele.com (Hitoshi Kashima)
//	2011/12/18	初版
//	2017/6/17	Ver1.02	changed		ODBCドライバ経由のCSV読込みを手動読込みへ変更
//						fixed		通信処理中にボタンが無効になるよう修正
//
using System;
using System.Data;
using System.IO;
using System.Windows.Forms;

namespace METCMonitor
{
    public partial class FormMain : Form
    {
		DataTable dt_comports = new DataTable();
		DataTable dt_gatedb = new DataTable();
		METC metc;

		// 初期化ルーチン
		public FormMain()
        {
            InitializeComponent();

			// COMポート一覧のDataTableを作成
			dt_comports.Columns.Add("DeviceID", typeof(string));
			dt_comports.Columns.Add("Caption", typeof(string));

			// 搭載COMポートを列挙する
			foreach (COMPortInfo p in COMPortInfo.GetCOMPortsInfo())
			{
				DataRow row = dt_comports.NewRow();
				row["DeviceID"] = (string)p.Name;
				row["Caption"] = (string)p.Description;
				dt_comports.Rows.Add(row);
			}

			// ComboBoxに登録
			comboBox1.DataSource = dt_comports;
			comboBox1.DisplayMember = "Caption";
			comboBox1.ValueMember = "DeviceID";

			statusStrip1.Items.Add(new ToolStripStatusLabel());
			this.Text = Application.ProductName + " 1.02";
		}

		// フォームロード処理
		private void FormMain_Load(object sender, EventArgs e)
        {
			// METCクラスの準備
			metc = new METC();
			metc.Event += new METC.METCEventHandler(metc_Event);

			// IC名データーCSVファイルを DataTableに読み込む
			string CurDir = System.IO.Directory.GetCurrentDirectory();

			try {
				dt_gatedb = ConvertCSVtoDataTable(CurDir + "\\料金所番号.csv");
			}
			catch(Exception ex){
				MessageBox.Show("料金所番号.csvが読み込めません。\r\r" + ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
        }

		// 通信クラスからのイベントデレゲート
		delegate void metc_eventdelegate(METC.ETCStatus f, int val);

		void metc_Event(object sender, METC.ETCStatus f, int val)
		{
			if(InvokeRequired)				// 子スレッドからであれば Invokeする
				Invoke( new metc_eventdelegate(PutEventMessage), new object[] { f, val });
		}

		// 通信クラスからのメッセージ処理
		void PutEventMessage(METC.ETCStatus f, int val)
		{
			lbDisp.Text = "";
			lbFrom.Text = "";
			lbTo.Text = "";

			switch (f)
			{
				case METC.ETCStatus.UnKnownMessage:
					lbDisp.Text = "？ " + (val >> 8).ToString("X2") + " - " + (val & 0x0ff).ToString("X2");
					statusStrip1.Items[0].Text = "";
					break;

				case METC.ETCStatus.TimeOut:
					lbDisp.Text = "通信エラー";
					break;

				case METC.ETCStatus.CardInsert:
					lbDisp.Text = "カード挿入";
					statusStrip1.Items[0].Text = "";
					break;

				case METC.ETCStatus.CardError:
					lbDisp.Text = "カードエラー";
					break;

				case METC.ETCStatus.Error05:
					lbDisp.Text = "エラー０５";
					break;

				case METC.ETCStatus.CardRemove:
					lbDisp.Text = "カードイジェクト";
					break;

				case METC.ETCStatus.GateIn:
					lbDisp.Text = "ゲート入口";
					statusStrip1.Items[0].Text = "";
					break;

				case METC.ETCStatus.GateOut:
					lbDisp.Text = "ゲート出口";
					statusStrip1.Items[0].Text = "";
					break;

				case METC.ETCStatus.Error01:
					lbDisp.Text = "！！カード確認！！";
					statusStrip1.Items[0].Text = "";
					break;

				case METC.ETCStatus.GateErr:
					lbDisp.Text = "ゲート通過エラー";
					statusStrip1.Items[0].Text = "";
					break;

				case METC.ETCStatus.PaidInfo:
					lbDisp.Text = "支払　" + val.ToString() + "円";
					statusStrip1.Items[0].Text = "";
					break;

				case METC.ETCStatus.OnCard:
					statusStrip1.Items[0].Text = "カードあり";
					break;

				case METC.ETCStatus.NoCard:
					statusStrip1.Items[0].Text = "カードなし";
					break;

				case METC.ETCStatus.CmdError:
					statusStrip1.Items[0].Text = "-";
					break;
			}
		}

		// 接続ボタン処理
		private void btn_Connect_Click(object sender, EventArgs e)
		{
			if (!metc.IsOpen)
			{
				if (comboBox1.SelectedValue == null)
				{
					MessageBox.Show("通信ポートを指定して下さい", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
					return;
				}
				comboBox1.Enabled = false;
				btn_Connect.Enabled = false;
				// ETC車載器との通信オープン
				if (metc.OpenPort(comboBox1.SelectedValue.ToString()) == METC.ETCStatus.TimeOut)
				{
					statusStrip1.Items[0].Text = "通信エラー";
					CloseConnection();
					return;
				}

				statusStrip1.Items[0].Text = "";
				tbMachineID.Text = metc.machineID;
				tbClassID.Text = metc.classID;
				tbModelName.Text = metc.modelName;
				tb日付.Text = metc.setupDate;

				btn_Connect.Enabled = true;
				btn_Connect.Text = "切断";
				btn_ShowHist.Enabled = true;
				btn_Check.Enabled = true;
				btnChkPaid.Enabled = true;
			}

			else
			{
				CloseConnection();
			}
		}

		// 切断処理
		void CloseConnection()
		{
			metc.ClosePort();
			comboBox1.Enabled = true;
			btn_Connect.Enabled = true;
			btn_Check.Enabled = false;
			btnChkPaid.Enabled = false;
			btn_ShowHist.Enabled = false;
			btn_Connect.Text = "接続";
			tbClassID.Text = ""; tbMachineID.Text = ""; tbModelName.Text = ""; tb日付.Text = "";
		}

		// 車種番号のラベル
		String[] klabel = { "不明", "普通", "大型", "特大", "中型", "軽二" };

		// 履歴ボタン処理
		private void btn_ShowHist_Click(object sender, EventArgs e)
		{
			// 履歴フォームを生成
			FormHistory f = new FormHistory();

			btn_ShowHist.Enabled = false;
			btn_Check.Enabled = false;
			btnChkPaid.Enabled = false;

			statusStrip1.Items[0].Text = "履歴の読込中...";
			statusStrip1.Update();

			// 履歴の照会
			switch (metc.ReadHistory())
			{
				case METC.ETCStatus.HistAvail:
					metc.ReadHistory();
					break;

				case METC.ETCStatus.History:
					break;

				case METC.ETCStatus.NoHist:
					statusStrip1.Items[0].Text = "履歴がありません";
					btn_ShowHist.Enabled = true;
					btn_Check.Enabled = true;
					btnChkPaid.Enabled = true;
					return;

				default:
					statusStrip1.Items[0].Text = "読み込みエラー";
					btn_ShowHist.Enabled = true;
					btn_Check.Enabled = true;
					btnChkPaid.Enabled = true;
					return;
			}

			// 継続なしを受信するまで繰り返す
			int i = 1;
			METC.ETCStatus s = METC.ETCStatus.History;
			while (true)
			{
				if (s == METC.ETCStatus.NoMore)				// 継続なし
				{
					break;
				}

				else if (s == METC.ETCStatus.History)		// 履歴データー
				{
					// 料金所番号を漢字名称に変換する
					string ename = GateCode2Name(metc.HistoryData.inGateGroup, metc.HistoryData.inGateNumber);
					string oname = GateCode2Name(metc.HistoryData.outGateGroup, metc.HistoryData.outGateNumber);

					// 履歴フォームの DataGridViewに追加
					f.AddHistory(metc.HistoryData.HistoryDate, ename, oname, klabel[(int)metc.HistoryData.VehicleType], metc.HistoryData.cost);
					statusStrip1.Items[0].Text = "履歴の読込中..." + i.ToString();
					statusStrip1.Update();
					i++;

					s = metc.ReadNext();					// 継続読みだしコマンド
				}
				else
				{
					statusStrip1.Items[0].Text = "読み込みエラー";
					btn_ShowHist.Enabled = true;
					btn_Check.Enabled = true;
					btnChkPaid.Enabled = true;
					return;
				}

				Application.DoEvents();						// 読込中フォームが固まらないようにする
			}

			statusStrip1.Items[0].Text = "読み込み終了";
			f.ShowDialog();									// 履歴フォームを表示する
			statusStrip1.Items[0].Text = "";
			btn_ShowHist.Enabled = true;
			btn_Check.Enabled = true;
			btnChkPaid.Enabled = true;
		}

		// IC名DataTableから区分と番号で名称を検索する
		private String GateCode2Name(int group, int number)
		{
			string name;
			DataRow[] rows;

			rows = dt_gatedb.Select("[区分] = '" + group.ToString() + "' and [番号] = '" + number.ToString() + "'");
			if (rows.Length > 0)
				name = rows[0]["道路名"].ToString() + " " + rows[0]["料金所名"].ToString();
			else
				name = metc.HistoryData.inGateGroup.ToString() + "-" + metc.HistoryData.inGateNumber.ToString();

			return name;
		}

		// カードチェック処理
		private void btn_Check_Click(object sender, EventArgs e)
		{
			metc.CardCheck();
		}

		// 料金確認処理
		private void btnChkPaid_Click(object sender, EventArgs e)
		{
			btn_ShowHist.Enabled = false;
			btn_Check.Enabled = false;
			btnChkPaid.Enabled = false;

			// 履歴の照会
			METC.ETCStatus s = metc.ReadHistory();
			switch (s)
			{
				case METC.ETCStatus.History:		// いきなり履歴データーが返ってきた場合はそのまま使う
					break;

				case METC.ETCStatus.HistAvail:
					s = metc.ReadHistory();			// 履歴ありの場合、最後の履歴を読み出す
					break;

				case METC.ETCStatus.NoHist:
					statusStrip1.Items[0].Text = "履歴がありません";
					btn_ShowHist.Enabled = true;
					btn_Check.Enabled = true;
					btnChkPaid.Enabled = true;
					return;

				default:
					statusStrip1.Items[0].Text = "読み込みエラー";
					btn_ShowHist.Enabled = true;
					btn_Check.Enabled = true;
					btnChkPaid.Enabled = true;
					return;
			}

			if( s == METC.ETCStatus.History)		// 履歴の表示
			{
					lbFrom.Text = "自　　" + GateCode2Name(metc.HistoryData.inGateGroup, metc.HistoryData.inGateNumber);
					lbTo.Text = "至　　" + GateCode2Name(metc.HistoryData.outGateGroup, metc.HistoryData.outGateNumber);
					lbDisp.Text = metc.HistoryData.cost.ToString() + "円";
					statusStrip1.Items[0].Text = "";
			}
			btn_ShowHist.Enabled = true;
			btn_Check.Enabled = true;
			btnChkPaid.Enabled = true;
		}

		// CSVファイルの読み込み
		public static DataTable ConvertCSVtoDataTable(string strFilePath)
		{
			DataTable dt = new DataTable();
			using (StreamReader sr = new StreamReader(strFilePath, System.Text.Encoding.GetEncoding("Shift_JIS"))) {
				string[] headers = sr.ReadLine().Split(',');
				foreach (string header in headers) {
					dt.Columns.Add(header);
				}
				while (!sr.EndOfStream) {
					string[] rows = sr.ReadLine().Split(',');
					DataRow dr = dt.NewRow();
					if (rows.Length == headers.Length) {
						for (int i = 0; i < headers.Length; i++) {
							dr[i] = rows[i];
						}
						dt.Rows.Add(dr);
					}
				}
			}
			return dt;
		}
	}
}
